@@ -248,6 +248,22 @@ class MLOps():
             'Catch': self.get_catch(),
             'Parameters': getattr(config, 'model_input')
         }
+    def create_generic_definition(self, name, config, next_step, resource):
+        """
+        creates a generic definition
+            Arguments:
+                name (str): name of parameter in config module
+                config (module): configuration module
+                next_step (str): name of the next step
+                resource (str): name of sagemaker resource
+            Returns:
+                Generic definition
+        """
+        return {'Type': 'Task', 'Resource': f'arn:aws:states:::sagemaker:{resource}',
+            'Next': next_step, 
+            'Catch': self.get_catch(),
+            'Parameters': getattr(config, name)
+        }
     def create_transform_definition(self, name, config, next_step = 'GoToTesting'):
         """
         creates the TransformJob
@@ -267,6 +283,24 @@ class MLOps():
             'Catch': self.get_catch(),
             'Parameters': getattr(config, name)
         }
+    def create_endpoint_choice_definition(self, config):
+        """
+        creates a choice definition
+            Arguments:
+                config (module): configuration module
+            Returns:
+                Choice definition
+        """
+        return {'Type': 'Choice',
+            'Choices': [
+                    {
+                        'Variable': '$.endpoint_exists',
+                        'NumericEquals': 0,
+                        'Next': 'GoToCreateEndpoint'
+                    }
+            ],
+            'Default': 'GoToUpdateEndpoint'
+        }
 
     def create_step_functions_processing_definition(self, name, config):
         """
@@ -332,6 +366,29 @@ class MLOps():
             }
         }
         return definition
+    def create_step_functions_endpoint_definition(self, config):
+        """
+        creates step functions definition
+            Arguments:
+                config (module): configuration module
+            Returns:
+                state machine definition
+        """
+        # https://docs.aws.amazon.com/step-functions/latest/dg/connect-stepfunctions.html
+        definition = {'Comment':'MLOps state machine', 'StartAt': 'GoToEndpointConfig',
+            'States': {
+                'GoToEndpointConfig': self.create_manage_config_definition('EndpointConfig', config),
+                'EndpointConfig': self.create_generic_definition('endpoint_config_input', config, 'EndpointChoice', 'createEndpointConfig'),
+                'EndpointChoice': self.create_endpoint_choice_definition(config),
+                'GoToCreateEndpoint': self.create_manage_config_definition('CreateEndpoint', config),
+                'CreateEndpoint': self.create_generic_definition('endpoint_input', config, 'GoToEnd', 'createEndpoint'),
+                'GoToUpdateEndpoint': self.create_manage_config_definition('UpdateEndpoint', config),
+                'UpdateEndpoint': self.create_generic_definition('endpoint_input', config, 'GoToEnd', 'updateEndpoint'),
+                'GoToEnd': self.create_manage_config_definition('End', config),
+                'Failure': self.create_failure_definition()
+            }
+        }
+        return definition
     def create_step_functions_mlops_definition(self, config):
         """
         creates step functions definition
@@ -374,6 +431,7 @@ class MLOps():
         definitions['reporting'] = self.create_step_functions_processing_definition('reporting_input', config)
         definitions['modeling'] = self.create_step_functions_modeling_definition('tuning_input', config)
         definitions['prediction'] = self.create_step_functions_prediction_definition('reporting_input', config)
+        #definitions['endpoint'] = self.create_step_functions_endpoint_definition(config)
         definitions['mlops'] = self.create_step_functions_mlops_definition(config)
         for definition in definitions.keys():
             with open(f'{config.environment}.{definition}.definition.json', 'w') as fh:
@@ -424,4 +482,4 @@ class MLOps():
             stateMachineArn=self.calc_state_machine_arn(name, config),
             name=self.create_unique_identificator(name, config),
             input=input
-        ), name, input]
\ No newline at end of file
+        ), name, input]
