@@ -55,37 +55,42 @@ class TestService(unittest.TestCase, DataStorage):
         fh.close()
 
         df_created = self.ds.create_dataframe_from_dict(json_config)
-        self.assertTrue('environment' in df_created)
-        self.assertTrue('estimator_input.container.version' in df_created)
-        self.assertTrue('tuner_input.environment' in df_created)
+        self.assertTrue('service' in df_created)
+        self.assertTrue('container_input.container.version' in df_created)
+        self.assertTrue('last_output.container_input.container.version' in df_created)
 
-        df_created = self.ds.create_dataframe_from_dict(json_config, ['container', 'tuner_input'])
-        self.assertTrue('environment' in df_created)
-        self.assertTrue('estimator_input.container.version' in df_created)
-        self.assertFalse('tuner_input.environment' in df_created)
+        df_created = self.ds.create_dataframe_from_dict(json_config, ['container_input', 'service'])
+        self.assertFalse('service' in df_created)
+        self.assertFalse('container_input.container.version' in df_created)
+        self.assertTrue('last_output.container_input.container.version' in df_created)
 
-        df_created = self.ds.create_dataframe_from_dict(json_config, ['environment', 'tuner_input'])
-        self.assertFalse('environment' in df_created)
-        self.assertTrue('estimator_input.container.version' in df_created)
-        self.assertFalse('tuner_input.environment' in df_created)
+        df_created = self.ds.create_dataframe_from_dict(json_config, ['last_output', 'service'])
+        self.assertFalse('service' in df_created)
+        self.assertTrue('container_input.container.version' in df_created)
+        self.assertFalse('last_output.container_input.container.version' in df_created)
 
     def test_create_dataframe_from_py(self):
         df_created = self.ds.create_dataframe_from_py(config)
+        df_created_columns = df_created.columns.to_list()
         self.ds.local_save(df_created, self.tmp, 'config.csv', True)
         df_restored = self.ds.local_read(self.tmp, 'config.csv')
-        df_diff = pd.concat([df_created, df_restored]).drop_duplicates(keep=False)
-        self.assertTrue(df_diff.empty)
+        self.assertTrue(df_created_columns.sort() == df_restored.columns.to_list().sort())
+        self.assertTrue('service' in df_restored)
+        self.assertTrue('container_input.version' in df_restored)
 
-        df_created = self.ds.create_dataframe_from_py(config, ['container', 'ContinuousParameter', 'IntegerParameter', 'os', 'git', 'datetime', 'get_commit', 'create_ts', 'create_ts', 'slash_to_dash', 'tuner_input'])
-        df_restored = self.ds.local_read(self.tmp, 'config.csv')
-        df_diff = pd.concat([df_created, df_restored]).drop_duplicates(keep=False)
-        self.assertTrue(df_diff.empty)
+        black_list = ['version', 'os', 'git', 'datetime', 'definitions_config', 'get_git_details', 'create_ts', 'slash_to_dash']
+        df_created_columns_without_black_list = list(set(df_created_columns) - set(black_list))
+        df_created = self.ds.create_dataframe_from_py(config, black_list)
+        self.assertTrue(df_created_columns_without_black_list.sort() == df_created.columns.to_list().sort())
+        self.assertTrue('service' in df_created)
+        self.assertTrue('container_input.version' in df_created)
 
-        df_created = self.ds.create_dataframe_from_py(config, ['environment', 'ContinuousParameter', 'IntegerParameter', 'os', 'git', 'datetime', 'get_commit', 'create_ts', 'create_ts', 'slash_to_dash', 'tuner_input'])
-        self.ds.local_save(df_created, self.tmp, 'config_without_environment.csv', True)
-        df_restored = self.ds.local_read(self.tmp, 'config_without_environment.csv')
-        df_diff = pd.concat([df_created, df_restored]).drop_duplicates(keep=False)
-        self.assertTrue(df_diff.empty)
+        black_list = ['service', 'container_input', 'os', 'git', 'datetime', 'definitions_config', 'get_git_details', 'create_ts', 'slash_to_dash']
+        df_created_columns_without_black_list = list(set(df_created_columns) - set(black_list))
+        df_created = self.ds.create_dataframe_from_py(config, black_list)
+        self.assertTrue(df_created_columns_without_black_list.sort() == df_restored.columns.to_list().sort())
+        self.assertFalse('service' in df_created)
+        self.assertFalse('container_input.version' in df_created)
 
     def test_save_and_restore_test(self):
         data = {'target': [0, 1, 2, 3], 'identifier': ['a', 'b', 'c', 'd'], 'col_3': ['f', 's', 't', 'f']}
@@ -112,14 +117,27 @@ class TestService(unittest.TestCase, DataStorage):
         self.assertEqual(df_merged['identifier'][0], 'a')
         self.assertEqual(df_merged['col_3'][0], 's')
 
-        # print('prepared')
-        # print(df_prepared)
-        # df_diff = pd.concat([df_prepared, df_merged]).drop_duplicates(keep=False)
-        # print('df_diff')
-        # print(df_diff)
-        # print('concat')
-        # print(pd.concat([df_prepared, df_merged]))
-        # self.assertTrue(df_diff.empty)
- 
+    def test_convert_dtypes(self):
+        data = {'i': [0, 1, 2, 3], 'f': [0.1, 0.2, 0.3, 0.4], 'b': [True, False, True, False],
+                'is': ['0', '1', '2', '3'], 'fs': ['0.1', '0.2', '0.3', '0.4'], 'bs': ['True', 'False', 'True', 'False']}
+        df_prepared = pd.DataFrame.from_dict(data)
+        dtypes = {'is':'int', 'fs':'float', 'bs':'bool'}
+        self.assertEqual(df_prepared['i'].dtypes, 'int')
+        self.assertEqual(df_prepared['f'].dtypes, 'float64')
+        self.assertEqual(df_prepared['b'].dtypes, 'bool')
+        self.assertEqual(df_prepared['is'].sum(), '0123')
+        self.assertEqual(df_prepared['fs'].sum(), '0.10.20.30.4')
+        self.assertTrue(df_prepared['bs'][0])
+        self.assertTrue(df_prepared['bs'][1])
+        df_converted = self.ds.convert_dtypes(dtypes, df_prepared)
+        self.assertEqual(df_converted['i'].sum(), df_prepared['i'].sum())
+        self.assertEqual(df_converted['f'].sum(), df_prepared['f'].sum())
+        self.assertEqual(df_converted['b'][0], df_prepared['b'][0])
+        self.assertEqual(df_converted['b'][1], df_prepared['b'][1])
+        self.assertEqual(df_converted['is'].sum(), df_prepared['i'].sum())
+        self.assertEqual(df_converted['fs'].sum(), df_prepared['f'].sum())
+        self.assertEqual(df_converted['bs'][0], df_prepared['b'][0])
+        self.assertEqual(df_converted['bs'][1], df_prepared['b'][1])
+
 if __name__ == '__main__':
     unittest.main()
\ No newline at end of file
